@@ -25,14 +25,16 @@ from scipy.optimize.  It has a number of useful enhancements, including:
 
   * Many pre-built models for common lineshapes are included and ready to use.
 
-   version: 0.8.0
-   last update: 2014-Sep-21
-   License: MIT
+   version: 0.9.5
+   last update: 2016-Jul-26
+   License: BSD
    Authors:  Matthew Newville, The University of Chicago
              Till Stensitzki, Freie Universitat Berlin
              Daniel B. Allen, Johns Hopkins University
              Antonino Ingargiola, University of California, Los Angeles
 """
+import warnings
+import sys
 
 from .minimizer import minimize, Minimizer, MinimizerException
 from .parameter import Parameter, Parameters
@@ -49,5 +51,17 @@ from .uncertainties import ufloat, correlated_values
 
 ## versioneer code
 from ._version import get_versions
+
 __version__ = get_versions()['version']
 del get_versions
+
+# PY26 Depreciation Warning
+if sys.version_info[:2] == (2, 6):
+    warnings.warn('Support for Python 2.6.x was dropped with lmfit 0.9.5')
+
+# SCIPY 0.13 Depreciation Warning
+import scipy
+scipy_major, scipy_minor, scipy_other = scipy.__version__.split('.', 2)
+
+if int(scipy_major) == 0 and int(scipy_minor) < 15:
+    warnings.warn('Support for Scipy 0.14 was dropped with lmfit 0.9.5')
