@@ -2,7 +2,6 @@
 Parameter class
 """
 from __future__ import division
-from numpy import arcsin, cos, sin, sqrt, inf, nan
 import json
 from copy import deepcopy
 try:
@@ -10,8 +9,8 @@ try:
 except ImportError:
     from ordereddict import OrderedDict
 
+from numpy import array, arcsin, cos, sin, sqrt, inf, nan, isfinite
 from . import uncertainties
-
 from .asteval import Interpreter
 from .astutils import get_ast_names, valid_symbol_name
 
@@ -22,6 +21,42 @@ def check_ast_errors(expr_eval):
         expr_eval.raise_exception(None)
 
 
+def isclose(x, y, rtol=1e-5, atol=1e-8):
+    """
+    The truth whether two numbers are the same, within an absolute and
+    relative tolerance.
+
+    i.e. abs(`x` - `y`) <= (`atol` + `rtol` * absolute(`y`))
+
+    Parameters
+    ----------
+    x, y : float
+        Input values
+    rtol : float
+        The relative tolerance parameter (see Notes).
+    atol : float
+        The absolute tolerance parameter (see Notes).
+
+    Returns
+    -------
+    y : bool
+        Are `x` and `x` are equal within tolerance?
+    """
+    def within_tol(x, y, atol, rtol):
+        return abs(x - y) <= atol + rtol * abs(y)
+
+    xfin = isfinite(x)
+    yfin = isfinite(y)
+
+    # both are finite
+    if xfin and yfin:
+        return within_tol(x, y, atol, rtol)
+    elif x == y:
+        return True
+    else:
+        return False
+
+
 class Parameters(OrderedDict):
     """
     A dictionary of all the Parameters required to specify a fit model.
@@ -45,13 +80,24 @@ class Parameters(OrderedDict):
             self._asteval = Interpreter()
         self.update(*args, **kwds)
 
+    def copy(self):
+        """Parameters.copy() should always be a deepcopy"""
+        return self.__deepcopy__(None)
+
+    def __copy__(self, memo):
+        """Parameters.copy() should always be a deepcopy"""
+        self.__deepcopy__(memo)
+
     def __deepcopy__(self, memo):
-        _pars = Parameters()
+        """Parameters deepcopy needs to make sure that
+        asteval is available and that all individula
+        parameter objects are copied"""
+        _pars = Parameters(asteval=None)
 
         # find the symbols that were added by users, not during construction
         sym_unique = self._asteval.user_defined_symbols()
         unique_symbols = {key: deepcopy(self._asteval.symtable[key], memo)
-                          for key in sym_unique}
+                              for key in sym_unique}
         _pars._asteval.symtable.update(unique_symbols)
 
         # we're just about to add a lot of Parameter objects to the newly
@@ -85,7 +131,9 @@ class Parameters(OrderedDict):
         self._asteval.symtable[key] = par.value
 
     def __add__(self, other):
-        "add Parameters objects"
+        """
+        Add Parameters objects
+        """
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
         out = deepcopy(self)
@@ -95,7 +143,7 @@ class Parameters(OrderedDict):
 
     def __iadd__(self, other):
         """
-        add/assign Parameters objects
+        Add/assign Parameters objects
         """
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
@@ -103,6 +151,12 @@ class Parameters(OrderedDict):
         self.add_many(*params)
         return self
 
+    def __array__(self):
+        """
+        Parameters to array
+        """
+        return array([float(k) for k in self.values()])
+
     def __reduce__(self):
         """
         Required to pickle a Parameters instance.
@@ -124,24 +178,32 @@ class Parameters(OrderedDict):
 
         Parameters
         ----------
-        state : list
-            state[0] is a dictionary containing symbols that need to be
-            injected into _asteval.symtable
-            state[1:] are the Parameter instances to be added
-         is list of parameters
+        state : dict
+            state['unique_symbols'] is a dictionary containing symbols that
+            need to be injected into _asteval.symtable
+            state['params'] is a list of Parameter instances to be added
         """
-        # first add all the parameters
-        self.add_many(*state['params'])
-
-        # now update the Interpreter symbol table
+        # first update the Interpreter symbol table. This needs to be done
+        # first because Parameter's early in the list may depend on later
+        # Parameter's. This leads to problems because add_many eventually leads
+        # to a Parameter value being retrieved with _getval, which, if the
+        # dependent value hasn't already been added to the symtable, leads to
+        # an Error. Another way of doing this would be to remove all the expr
+        # from the Parameter instances before they get added, then to restore
+        # them.
         self._asteval.symtable.update(state['unique_symbols'])
 
+        # then add all the parameters
+        self.add_many(*state['params'])
+
     def update_constraints(self):
         """
         Update all constrained parameters, checking that dependencies are
         evaluated as needed.
         """
-        _updated = [name for name,par in self.items() if par._expr is None]
+        requires_update = set(name for name, par in self.items()
+                              if par._expr is not None)
+        updated_tracker = set(requires_update)
 
         def _update_param(name):
             """
@@ -150,24 +212,17 @@ class Parameters(OrderedDict):
             this first updates (recursively) all parameters on which
             the parameter depends (using the 'deps' field).
             """
-            # Has this param already been updated?
-            if name in _updated:
-                return
             par = self.__getitem__(name)
             if par._expr_eval is None:
                 par._expr_eval = self._asteval
-            if par._expr is not None:
-                par.expr = par._expr
-            if par._expr_ast is not None:
-                for dep in par._expr_deps:
-                    if dep in self.keys():
-                        _update_param(dep)
+            for dep in par._expr_deps:
+                if dep in updated_tracker:
+                    _update_param(dep)
             self._asteval.symtable[name] = par.value
-            _updated.append(name)
+            updated_tracker.discard(name)
 
-        for name in self.keys():
-            if name not in _updated:
-                _update_param(name)
+        for name in requires_update:
+            _update_param(name)
 
     def pretty_repr(self, oneline=False):
         if oneline:
@@ -178,10 +233,48 @@ class Parameters(OrderedDict):
         s += "    })\n"
         return s
 
-    def pretty_print(self, oneline=False):
-        print(self.pretty_repr(oneline=oneline))
+    def pretty_print(self, oneline=False, colwidth=8, precision=4, fmt='g',
+                     columns=['value', 'min', 'max', 'stderr', 'vary', 'expr']):
+        """Pretty-print parameters data.
 
-    def add(self, name, value=None, vary=True, min=None, max=None, expr=None):
+        Parameters
+        ----------
+        oneline : boolean
+            If True prints a one-line parameters representation. Default False.
+        colwidth : int
+            column width for all except the first (i.e. name) column.
+        columns : list of strings
+            list of columns names to print. All values must be valid
+            :class:`Parameter` attributes.
+        fmt : string
+            single-char numeric formatter. Valid values: 'f' floating point,
+            'g' floating point and exponential, 'e' exponential.
+        precision : int
+            number of digits to be printed after floating point.
+        """
+        if oneline:
+            print(self.pretty_repr(oneline=oneline))
+            return
+
+        name_len = max(len(s) for s in self)
+        allcols = ['name'] + columns
+        title = '{:{name_len}} ' + len(columns) * ' {:>{n}}'
+        print(title.format(*allcols, name_len=name_len, n=colwidth).title())
+        numstyle = '{%s:>{n}.{p}{f}}'  # format for numeric columns
+        otherstyles = dict(name='{name:<{name_len}} ', stderr='{stderr!s:>{n}}',
+                           vary='{vary!s:>{n}}', expr='{expr!s:>{n}}')
+        line = ' '.join([otherstyles.get(k, numstyle % k) for k in allcols])
+        for name, values in sorted(self.items()):
+            pvalues = {k: getattr(values, k) for k in columns}
+            pvalues['name'] = name
+            # stderr is a special case: it is either numeric or None (i.e. str)
+            if 'stderr' in columns and pvalues['stderr'] is not None:
+                pvalues['stderr'] = (numstyle % '').format(
+                    pvalues['stderr'], n=colwidth, p=precision, f=fmt)
+            print(line.format(name_len=name_len, n=colwidth, p=precision, f=fmt,
+                              **pvalues))
+
+    def add(self, name, value=None, vary=True, min=-inf, max=inf, expr=None):
         """
         Convenience function for adding a Parameter:
 
@@ -335,9 +428,9 @@ class Parameter(object):
     vary : bool
         Whether the Parameter is fixed during a fit.
     min : float
-        Lower bound for value (None = no lower bound).
+        Lower bound for value (None or -inf means no lower bound).
     max : float
-        Upper bound for value (None = no upper bound).
+        Upper bound for value (None or inf means no upper bound).
     expr : str
         An expression specifying constraints for the parameter.
     stderr : float
@@ -347,7 +440,7 @@ class Parameter(object):
         Of the form `{'decay': 0.404, 'phase': -0.020, 'frequency': 0.102}`
     """
     def __init__(self, name=None, value=None, vary=True,
-                 min=None, max=None, expr=None):
+                 min=-inf, max=inf, expr=None):
         """
         Parameters
         ----------
@@ -358,9 +451,9 @@ class Parameter(object):
         vary : bool, optional
             Whether the Parameter is fixed during a fit.
         min : float, optional
-            Lower bound for value (None = no lower bound).
+            Lower bound for value (None or -inf means no lower bound).
         max : float, optional
-            Upper bound for value (None = no upper bound).
+            Upper bound for value (None or inf means no upper bound).
         expr : str, optional
             Mathematical expression used to constrain the value during the fit.
         """
@@ -381,7 +474,7 @@ class Parameter(object):
         self.from_internal = lambda val: val
         self._init_bounds()
 
-    def set(self, value=None, vary=None, min=None, max=None, expr=None):
+    def set(self, value=None, vary=None, min=-inf, max=inf, expr=None):
         """
         Set or update Parameter attributes.
 
@@ -405,23 +498,32 @@ class Parameter(object):
             self._val = value
         if vary is not None:
             self.vary = vary
-        if min is not None:
-            self.min = min
-        if max is not None:
-            self.max = max
+        if min is None:
+            min = -inf
+        if max is None:
+            max = inf
+        self.min = min
+        self.max = max
 
     def _init_bounds(self):
         """make sure initial bounds are self-consistent"""
-        #_val is None means - infinity.
+        # _val is None means - infinity.
+        if self.max is None:
+            self.max = inf
+        if self.min is None:
+            self.min = -inf
         if self._val is not None:
-            if self.max is not None and self._val > self.max:
+            if self.min > self.max:
+                self.min, self.max = self.max, self.min
+            if isclose(self.min, self.max, atol=1e-13, rtol=1e-13):
+                raise ValueError("Parameter '%s' has min == max" % self.name)
+
+            if self._val > self.max:
                 self._val = self.max
-            if self.min is not None and self._val < self.min:
+            if self._val < self.min:
                 self._val = self.min
-        elif self.min is not None and self._expr is None:
+        elif self._expr is None:
             self._val = self.min
-        elif self.max is not None and self._expr is None:
-            self._val = self.max
         self.setup_bounds()
 
     def __getstate__(self):
@@ -445,13 +547,13 @@ class Parameter(object):
             s.append("'%s'" % self.name)
         sval = repr(self._getval())
         if not self.vary and self._expr is None:
-            sval = "value=%s (fixed)" % (sval)
+            sval = "value=%s (fixed)" % sval
         elif self.stderr is not None:
             sval = "value=%s +/- %.3g" % (sval, self.stderr)
         s.append(sval)
         s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
         if self._expr is not None:
-            s.append("expr='%s'" % (self.expr))
+            s.append("expr='%s'" % self.expr)
         return "<Parameter %s>" % ', '.join(s)
 
     def setup_bounds(self):
@@ -473,19 +575,23 @@ class Parameter(object):
         the external, user-expected value).   This internal value should
         actually be used in a fit.
         """
-        if self.min in (None, -inf) and self.max in (None, inf):
+        if self.min is None:
+            self.min = -inf
+        if self.max is None:
+            self.max = inf
+        if self.min == -inf and self.max == inf:
             self.from_internal = lambda val: val
-            _val  = self._val
-        elif self.max in (None, inf):
+            _val = self._val
+        elif self.max == inf:
             self.from_internal = lambda val: self.min - 1.0 + sqrt(val*val + 1)
-            _val  = sqrt((self._val - self.min + 1.0)**2 - 1)
-        elif self.min in (None, -inf):
+            _val = sqrt((self._val - self.min + 1.0)**2 - 1)
+        elif self.min == -inf:
             self.from_internal = lambda val: self.max + 1 - sqrt(val*val + 1)
-            _val  = sqrt((self.max - self._val + 1.0)**2 - 1)
+            _val = sqrt((self.max - self._val + 1.0)**2 - 1)
         else:
             self.from_internal = lambda val: self.min + (sin(val) + 1) * \
                                  (self.max - self.min) / 2.0
-            _val  = arcsin(2*(self._val - self.min)/(self.max - self.min) - 1)
+            _val = arcsin(2*(self._val - self.min)/(self.max - self.min) - 1)
         return _val
 
     def scale_gradient(self, val):
@@ -495,16 +601,15 @@ class Parameter(object):
         scaling factor for gradient the according to Minuit-style
         transformation.
         """
-        if self.min in (None, -inf) and self.max in (None, inf):
+        if self.min == -inf and self.max == inf:
             return 1.0
-        elif self.max in (None, inf):
+        elif self.max == inf:
             return val / sqrt(val*val + 1)
-        elif self.min in (None, -inf):
+        elif self.min == -inf:
             return -val / sqrt(val*val + 1)
         else:
             return cos(val) * (self.max - self.min) / 2.0
 
-
     def _getval(self):
         """get value, with bounds applied"""
 
@@ -516,53 +621,48 @@ class Parameter(object):
         # becomes stale if parameter.expr is not None.
         if (isinstance(self._val, uncertainties.Variable)
             and self._val is not nan):
+
             try:
                 self.value = self._val.nominal_value
             except AttributeError:
                 pass
         if not self.vary and self._expr is None:
             return self._val
-        if not hasattr(self, '_expr_eval'):
-            self._expr_eval = None
-        if not hasattr(self, '_expr_ast'):
-            self._expr_ast = None
-        if self._expr_ast is None and self._expr is not None:
-            self.__set_expression(self._expr)
 
-        if self._expr_ast is not None and self._expr_eval is not None:
-            if not self._delay_asteval:
-                self.value = self._expr_eval(self._expr_ast)
-                check_ast_errors(self._expr_eval)
+        if self._expr is not None:
+            if self._expr_ast is None:
+                self.__set_expression(self._expr)
 
-        if self.min is None:
-            self.min = -inf
-        if self.max is None:
-            self.max = inf
-        if self.max < self.min:
-            self.max, self.min = self.min, self.max
-        if (abs((1.0 * self.max - self.min)/
-                max(abs(self.max), abs(self.min), 1.e-13)) < 1.e-13):
-            raise ValueError("Parameter '%s' has min == max" % self.name)
-        try:
-            self.value = max(self.min, min(self._val, self.max))
-        except(TypeError, ValueError):
-            self.value = nan
+            if self._expr_eval is not None:
+                if not self._delay_asteval:
+                    self.value = self._expr_eval(self._expr_ast)
+                    check_ast_errors(self._expr_eval)
+
+        v = self._val
+        if v > self.max:
+            v = self.max
+        if v < self.min:
+            v = self.min
+        self.value = self._val = v
         return self._val
 
     def set_expr_eval(self, evaluator):
-        "set expression evaluator instance"
+        """set expression evaluator instance"""
         self._expr_eval = evaluator
 
     @property
     def value(self):
-        "The numerical value of the Parameter, with bounds applied"
+        """The numerical value of the Parameter, with bounds applied"""
         return self._getval()
 
     @value.setter
     def value(self, val):
-        "Set the numerical Parameter value."
+        """
+        Set the numerical Parameter value.
+        """
         self._val = val
-        if not hasattr(self, '_expr_eval'):  self._expr_eval = None
+        if not hasattr(self, '_expr_eval'):
+            self._expr_eval = None
         if self._expr_eval is not None:
             self._expr_eval.symtable[self.name] = val
 
@@ -587,139 +687,143 @@ class Parameter(object):
         self._expr = val
         if val is not None:
             self.vary = False
-        if not hasattr(self, '_expr_eval'):  self._expr_eval = None
-        if val is None: self._expr_ast = None
+        if not hasattr(self, '_expr_eval'):
+            self._expr_eval = None
+        if val is None:
+            self._expr_ast = None
         if val is not None and self._expr_eval is not None:
             self._expr_ast = self._expr_eval.parse(val)
             check_ast_errors(self._expr_eval)
             self._expr_deps = get_ast_names(self._expr_ast)
 
+    def __array__(self):
+        """array"""
+        return array(float(self._getval()))
+
     def __str__(self):
-        "string"
+        """string"""
         return self.__repr__()
 
     def __abs__(self):
-        "abs"
+        """abs"""
         return abs(self._getval())
 
     def __neg__(self):
-        "neg"
+        """neg"""
         return -self._getval()
 
     def __pos__(self):
-        "positive"
+        """positive"""
         return +self._getval()
 
     def __nonzero__(self):
-        "not zero"
+        """not zero"""
         return self._getval() != 0
 
     def __int__(self):
-        "int"
+        """int"""
         return int(self._getval())
 
-    def __long__(self):
-        "long"
-        return long(self._getval())
-
     def __float__(self):
-        "float"
+        """float"""
         return float(self._getval())
 
     def __trunc__(self):
-        "trunc"
+        """trunc"""
         return self._getval().__trunc__()
 
     def __add__(self, other):
-        "+"
+        """+"""
         return self._getval() + other
 
     def __sub__(self, other):
-        "-"
+        """-"""
         return self._getval() - other
 
     def __div__(self, other):
-        "/"
+        """/"""
         return self._getval() / other
     __truediv__ = __div__
 
     def __floordiv__(self, other):
-        "//"
+        """//"""
         return self._getval() // other
 
     def __divmod__(self, other):
-        "divmod"
+        """divmod"""
         return divmod(self._getval(), other)
 
     def __mod__(self, other):
-        "%"
+        """%"""
         return self._getval() % other
 
     def __mul__(self, other):
-        "*"
+        """*"""
         return self._getval() * other
 
     def __pow__(self, other):
-        "**"
+        """**"""
         return self._getval() ** other
 
     def __gt__(self, other):
-        ">"
+        """>"""
         return self._getval() > other
 
     def __ge__(self, other):
-        ">="
+        """>="""
         return self._getval() >= other
 
     def __le__(self, other):
-        "<="
+        """<="""
         return self._getval() <= other
 
     def __lt__(self, other):
-        "<"
+        """<"""
         return self._getval() < other
 
     def __eq__(self, other):
-        "=="
+        """=="""
         return self._getval() == other
+
     def __ne__(self, other):
-        "!="
+        """!="""
         return self._getval() != other
 
     def __radd__(self, other):
-        "+ (right)"
+        """+ (right)"""
         return other + self._getval()
 
     def __rdiv__(self, other):
-        "/ (right)"
+        """/ (right)"""
         return other / self._getval()
     __rtruediv__ = __rdiv__
 
     def __rdivmod__(self, other):
-        "divmod (right)"
+        """divmod (right)"""
         return divmod(other, self._getval())
 
     def __rfloordiv__(self, other):
-        "// (right)"
+        """// (right)"""
         return other // self._getval()
 
     def __rmod__(self, other):
-        "% (right)"
+        """% (right)"""
         return other % self._getval()
 
     def __rmul__(self, other):
-        "* (right)"
+        """* (right)"""
         return other * self._getval()
 
     def __rpow__(self, other):
-        "** (right)"
+        """** (right)"""
         return other ** self._getval()
 
     def __rsub__(self, other):
-        "- (right)"
+        """- (right)"""
         return other - self._getval()
 
+
 def isParameter(x):
-    "test for Parameter-ness"
+    """Test for Parameter-ness"""
     return (isinstance(x, Parameter) or
             x.__class__.__name__ == 'Parameter')
