@@ -1,7 +1,7 @@
 import unittest
 import warnings
 import nose
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose, assert_raises
 from numpy.testing.decorators import knownfailureif
 import numpy as np
 
@@ -57,12 +57,12 @@ class CommonTests(object):
         assert_results_close(result.values, self.true_values())
 
         # Pass inidividual Parameter objects as kwargs.
-        kwargs = {name: p for name, p in params.items()}
+        kwargs = dict((name, p) for name, p in params.items())
         result = self.model.fit(self.data, x=self.x, **kwargs)
         assert_results_close(result.values, self.true_values())
 
         # Pass guess values (not Parameter objects) as kwargs.
-        kwargs = {name: p.value for name, p in params.items()}
+        kwargs = dict((name, p.value) for name, p in params.items())
         result = self.model.fit(self.data, x=self.x, **kwargs)
         assert_results_close(result.values, self.true_values())
 
@@ -119,7 +119,6 @@ class CommonTests(object):
         # Check eval() output against init_fit and best_fit.
         pars = self.model.make_params(**self.guess())
         result = self.model.fit(self.data, pars, x=self.x)
-
         assert_allclose(result.eval(x=self.x, **result.values),
                         result.best_fit)
         assert_allclose(result.eval(x=self.x, **result.init_values),
@@ -132,7 +131,8 @@ class CommonTests(object):
 
         # Check that the independent variable is respected.
         short_eval = result.eval(x=np.array([0, 1, 2]), **result.values)
-        self.assertEqual(len(short_eval), 3)
+        if hasattr(short_eval, '__len__'):
+            self.assertEqual(len(short_eval), 3)
 
     def test_data_alignment(self):
         _skip_if_no_pandas()
@@ -218,7 +218,11 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
     def test_lists_become_arrays(self):
         # smoke test
         self.model.fit([1, 2, 3], x=[1, 2, 3], **self.guess())
-        self.model.fit([1, 2, None, 3], x=[1, 2, 3, 4], **self.guess())
+        assert_raises(ValueError,
+                      self.model.fit,
+                      [1, 2, None, 3],
+                      x=[1, 2, 3, 4],
+                      **self.guess())
 
     def test_missing_param_raises_error(self):
 
@@ -429,7 +433,7 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
 
         mx = (m1 + m2)
         params = mx.make_params()
-        param_values = {name: p.value for name, p in params.items()}
+        param_values = dict((name, p.value) for name, p in params.items())
         self.assertEqual(param_values['p1_amplitude'], 1)
         self.assertEqual(param_values['p2_amplitude'], 2)
 
@@ -448,7 +452,7 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
 
         m = m1 + m2
 
-        param_values = {name: p.value for name, p in params.items()}
+        param_values = dict((name, p.value) for name, p in params.items())
         self.assertTrue(param_values['m1_intercept'] < -0.0)
         self.assertEqual(param_values['m2_amplitude'], 1)
 
@@ -581,5 +585,4 @@ class TestComplexConstant(CommonTests, unittest.TestCase):
         self.guess = lambda: dict(re=2,im=2)
         self.model_constructor = models.ComplexConstantModel
         super(TestComplexConstant, self).setUp()
-
 #
