@@ -1,6 +1,7 @@
 from __future__ import print_function
 from lmfit import Parameters, Parameter
-from numpy.testing import assert_, assert_almost_equal
+from lmfit.parameter import isclose
+from numpy.testing import assert_, assert_almost_equal, assert_equal
 import unittest
 from copy import deepcopy
 import numpy as np
@@ -20,6 +21,27 @@ class TestParameters(unittest.TestCase):
         assert_almost_equal(self.params['c'].value,
                             2 * self.params['a'].value)
 
+    def test_copy(self):
+        # check simple Parameters.copy() does not fail
+        # on non-trivial Parameters
+        p1 = Parameters()
+        p1.add('t', 2.0, min=0.0, max=5.0)
+        p1.add('x', 10.0)
+        p1.add('y', expr='x*t + sqrt(t)/3.0')
+
+        p2 = p1.copy()
+        assert(isinstance(p2, Parameters))
+        assert('t' in p2)
+        assert('y' in p2)
+        assert(p2['t'].max < 6.0)
+        assert(np.isinf(p2['x'].max) and p2['x'].max > 0)
+        assert(np.isinf(p2['x'].min) and p2['x'].min < 0)
+        assert('sqrt(t)' in p2['y'].expr )
+        assert(p2._asteval is not None)
+        assert(p2._asteval.symtable is not None)
+        assert((p2['y'].value > 20) and (p2['y'].value < 21))
+
+
     def test_deepcopy(self):
         # check that a simple copy works
         b = deepcopy(self.params)
@@ -64,6 +86,9 @@ class TestParameters(unittest.TestCase):
         p['a'] = Parameter('a', 10, True)
         p['b'] = Parameter('b', 10, True, 0, 20)
 
+        assert_equal(p['b'].min, 0)
+        assert_equal(p['b'].max, 20)
+
         p['a'].expr = '2 * b'
         assert_almost_equal(p['a'].value, 20)
 
@@ -102,6 +127,26 @@ class TestParameters(unittest.TestCase):
         # now test if the asteval machinery survived
         assert_(q._asteval.symtable['abc'] == '2 * 3.142')
 
+        # check that unpickling of Parameters is not affected by expr that
+        # refer to Parameter that are added later on. In the following
+        # example var_0.expr refers to var_1, which is a Parameter later
+        # on in the Parameters OrderedDict.
+        p = Parameters()
+        p.add('var_0', value=1)
+        p.add('var_1', value=2)
+        p['var_0'].expr = 'var_1'
+        pkl = pickle.dumps(p)
+        q = pickle.loads(pkl)
+
+    def test_isclose(self):
+        assert_(isclose(1., 1+1e-5, atol=1e-4, rtol=0))
+        assert_(not isclose(1., 1+1e-5, atol=1e-6, rtol=0))
+        assert_(isclose(1e10, 1.00001e10, rtol=1e-5, atol=1e-8))
+        assert_(not isclose(0, np.inf))
+        assert_(not isclose(-np.inf, np.inf))
+        assert_(isclose(np.inf, np.inf))
+        assert_(not isclose(np.nan, np.nan))
+
 
 if __name__ == '__main__':
     unittest.main()
